@@ -34,7 +34,7 @@ from parsedmarc.utils import is_outlook_msg, convert_outlook_msg
 from parsedmarc.utils import parse_email
 from parsedmarc.utils import timestamp_to_human, human_timestamp_to_datetime
 
-__version__ = "8.2.0"
+__version__ = "8.3.0"
 
 formatter = logging.Formatter(
     fmt='%(levelname)8s:%(filename)s:%(lineno)d:%(message)s',
@@ -230,8 +230,9 @@ def parse_aggregate_report_xml(xml, ip_db_path=None, offline=False,
         xmltodict.parse(xml)["feedback"]
     except Exception as e:
         errors.append("Invalid XML: {0}".format(e.__str__()))
-        tree = etree.parse(BytesIO(xml.encode('utf-8')),
-                           etree.XMLParser(recover=True))
+        tree = etree.parse(
+            BytesIO(xml.encode('utf-8')),
+            etree.XMLParser(recover=True, resolve_entities=False))
         s = etree.tostring(tree)
         xml = '' if s is None else s.decode('utf-8')
 
@@ -389,9 +390,11 @@ def extract_xml(input_):
         file_object.close()
 
     except UnicodeDecodeError:
+        file_object.close()
         raise InvalidAggregateReport("File objects must be opened in binary "
                                      "(rb) mode")
     except Exception as error:
+        file_object.close()
         raise InvalidAggregateReport(
             "Invalid archive file: {0}".format(error.__str__()))
 
@@ -1223,6 +1226,7 @@ def watch_inbox(mailbox_connection: MailboxConnection,
     """
     Watches the mailbox for new messages and
       sends the results to a callback function
+
     Args:
         mailbox_connection: The mailbox connection object
         callback: The callback function to receive the parsing results
